# boog - a commandline frontend for bug reporting tool
#
# Copyright (C) 2009-2010 Red Hat Inc.
# Author: Kushal Das <kushal@redhat.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
# the full text of the license.


import os
import sys
import tempfile


from snack import *
from boogreporter import BoogReporter

from ConfigParser import ConfigParser
username = None
password = None

def get_login_details():
    """
    Get the login details from /etc/boog.conf
    """
    cf = ConfigParser()

    username = ''
    password = ''

    if os.path.exists('/etc/boog.conf'):
        cf.readfp(open('/etc/boog.conf'))
        username = cf.get("details",'username')
        password = cf.get("details",'password')
    return (username, password)



def show_authwindow(screen,username,password, wrong=False):
    """
    Show the authentication window
    """
    t = TextboxReflowed(40,'After pressing Ok please wait for sometime while we authenticate\n\n\n')
    g = Grid(2, 2)
    bb = ButtonBar(screen, (("Ok", "ok"), ("Cancel", "cancel")))
    user_field = Entry(30,username)
    password_field = Entry(30,password,password=1)
    user_label = Label('Username')
    g.setField(user_label,0,0,anchorLeft=1)
    g.setField(user_field,1,0,anchorLeft=1)
    password_label = Label('Password')
    g.setField(password_label,0,1,anchorLeft=1)
    g.setField(password_field,1,1,anchorLeft=1)
    #g.add(bb,0,2)
    main_grid = Grid(1,5)
    if wrong:
        l = Label('Wrong username/password\n')
        main_grid.setField(l,0,0,anchorLeft=1)
    main_grid.setField(t,0,1,anchorLeft=1)
    main_grid.setField(g,0,2,anchorLeft=1)
    main_grid.setField(Label(''),0,3,anchorLeft=1)
    main_grid.setField(bb,0,4,anchorLeft=1)

    main_grid.place(1,1)

    screen.gridWrappedWindow(main_grid,'Boog')

    form = Form('Bugzilla authentication')
    if wrong:
        form.add(l)
    form.add(t)
    form.add(bb)
    form.add(user_field)
    form.add(password_field)
    form.add(user_label)
    form.add(password_label)

    res = form.run()
    res = bb.buttonPressed(res)

    if res == 'cancel':
        return ("","")
        screen.refresh()
        screen.finish()
        sys.exit(0)

    username = user_field.value()
    password = password_field.value()
    return (username, password)

def get_summary(screen,bz):
    """
    Get the summary message from the user
    """
    em = EntryWindow(screen,'Boog','RPM installed in your system: ' + bz.rpm_details + '\n\nWe will open up your fav editor after this. Please write the full description there then save and exit', [('Summary of your problem',bz.summary),],entryWidth=40,buttons=['Next', 'Back'])
    if em[0] == 'back':
        return 0
    else:
        bz.set_summary(em[1][0])
        return 2


def get_package_name(screen,bz):
    """
    Try to find the package name from the user
    """
    em = EntryWindow(screen,'Boog','Tell us the name of the application',[('Name of package',bz.name)],buttons=['Next', 'Cancel'])
    if em[0] == 'cancel':
        screen.finish()
        sys.exit(0)
    elif em[1][0]:
        bz.set_name(em[1][0])
        screen.refresh()
        data = bz.get_application_details()
        if not data:
            ButtonChoiceWindow(screen,'No Such package','Sorry, but we can not find any such package installed in your system', buttons=['Ok'])
            return 0
        elif type(data) == type([]):
            length = len(data)
            g = GridForm(screen, "Please select the correct version", 1, 2)
            li = Listbox(height = length+1, width = 40)
            for val in data:
                li.append(val,val)
            g.add(li, 0, 0)
            g.add(ButtonBar(screen,(('Ok',))), 0, 1)
            result = g.runOnce()
            bz.rpm_details = li.current()


        #return the page id for the next page
        return 1

    else:
        return 0

def get_description(bz):
    """
    Get the description of the problem from the user
    """
    f, name = tempfile.mkstemp()

    #Now save the old data (if any)
    f = open(name,'w')
    f.write(bz.user_desc)
    f.close()

    editor = os.getenv('EDITOR')
    if editor:
        os.system(editor + ' ' + name)
    else:
        os.system('vi '+ name)
    data = open(name,'r').read()
    return data

def show_final_data(screen,bz):
    """
    Show the final data before sending
    """
    g = GridForm(screen, "Boog", 1, 2)
    em = TextboxReflowed(55,bz.user_desc + '\n' + bz.auto_desc,maxHeight=20)

    g.add(em,0,0)
    bb = ButtonBar(screen, (("Submit", "submit"),('Cancel','cancel'),('Back','back')))
    g.add(bb,0,1)
    result=g.runOnce()
    if bb.buttonPressed(result) == 'back':
        return 1
    elif bb.buttonPressed(result) == 'submit':
        try:
            bz.file_a_new_bug()
            return show_result(screen, 'https://bugzilla.redhat.com/show_bug.cgi?id=' + \
                                    str(bz.bug.id))
        except:
            return show_result(screen, 'Sorry, but we had some problem while submitting the report. \
                                May be the component can not be found in Bugzilla.')
    else:
        screen.finish()
        sys.exit(0)

def show_result(screen, mes):
    """
    Show the URL
    """
    g = GridForm(screen, "Boog", 1, 2)
    em = TextboxReflowed(40, mes,maxHeight=20)
    bb = ButtonBar(screen, (("Done", "done"),))
    g.add(bb,0,1)

    g.add(em,0,0)
    result=g.runOnce()
    return False

def create_mainscreen():
    """
    Create a main screen and return it
    """
    screen = SnackScreen()
    screen.pushHelpLine('Press F12 to go to next screen')
    screen.drawRootText(0, 0, 'Boog, (C) 2009-2010 Red Hat, Inc.')
    screen.drawRootText(0, 1, 'Author: Kushal Das <kushal@redhat.com>')
    return screen



def main(screen):
    """
    The main method under loop
    """
    b = BoogReporter()

    username, password = get_login_details()

    wrong_pass_flag = False
    while True:
        if username and password:
            username, password = show_authwindow(screen,username,password, wrong_pass_flag)
        else:
            username, password = show_authwindow(screen,'','')
        if username == "":
            screen.popWindow()
            screen.finish()
            sys.exit(0)

    	try:
            if b.login(username, password):
                break
            wrong_pass_flag = True
            screen.popWindow()
        except:
            screen.popWindow()
            show_result(screen,"Please check your network")
            continue

    screen.popWindow()
    page_id = 0
    while True:
        if page_id == 0:
            page_id = get_package_name(screen,b)
            continue
        elif page_id == 1:
            page_id = get_summary(screen,b)
            continue
        elif page_id == 2:
            desc = get_description(b)
            screen.finish()
            screen = create_mainscreen()
            b.set_description(desc)
            b.create_final_description()
            page_id = 3
            continue
        elif page_id == 3:
            page_id = show_final_data(screen,b)
            if not page_id:
                break
            continue


    screen.finish()



if __name__ == '__main__':

    if os.getuid() != 0:
        print "Please run this application as superuser"
        sys.exit(0)

    screen = create_mainscreen()

    g = GridForm(screen, "Boog", 1, 2)
    em = TextboxReflowed(55, 'Welcome to Boog\n\nWe will help you to report a problem to Red Hat bug-tracking system, Red Hat bugzilla.\n\nYou can find it at https://bugzilla.redhat.com')

    g.add(em,0,0)
    bb = ButtonBar(screen, (("Next", "next"),))
    g.add(bb,0,1)
    result=g.runOnce()
    #print result
    main(screen)
